package org.code_blue.dmxcremote.v2;

import org.code_blue.dmxcremote.v2.classes.DMXConnection;
import org.code_blue.dmxcremote.v2.classes.TCPConnection;

import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

public class RemoteApp extends Application{
	
	private DMXConnection dmxConn=null;
	
	public enum HANDLE{
		CONNECTIONOK {
			void run(Bundle data,Context c){
				Log.d("DMX","sending broadcast: conn ok");
				c.sendBroadcast(new Intent("org.code_blue.dmxcremote.v2.CONNECTED"));
			} 
		},
		CONNECTIONFAILED {
			void run(Bundle data,Context c){
				Log.d("DMX","sending broadcast: conn failed");
				c.sendBroadcast(new Intent("org.code_blue.dmxcremote.v2.CONNECTIONFAILED"));
			} 
		},
		DISPLAYTOAST {
			void run(Bundle data,Context c){
				/*display toast*/
				Toast.makeText(c,data.getString("message"),Toast.LENGTH_SHORT).show();
			} 
		},
		DATAREADY {
			void run(Bundle data,Context c){
				Log.d("DMX","sending dtr broadcast");
				Intent i=new Intent("org.code_blue.dmxcremote.v2.DATAREADY");
				i.putExtra("data",data.getString("data"));
				c.sendBroadcast(i);
			}
		};
		
		abstract void run(Bundle data,Context c);
	}
	
	
	private Handler appLevelHandler=new Handler(){
		public void handleMessage(Message msg){
			if(!(msg.obj instanceof HANDLE)){
				return;
			}
			((HANDLE)msg.obj).run(msg.getData(),getApplicationContext());
		}
	};
	
	public DMXConnection getDMXConnection(){
		if(dmxConn==null){
			dmxConn=new TCPConnection(getApplicationHandler());
		}
		return dmxConn;
	}
	
	public Handler getApplicationHandler(){
		return appLevelHandler;
	}
}
