package org.code_blue.dmxcremote.v2.classes;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;

import org.code_blue.dmxcremote.v2.RemoteApp;

import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class TCPConnection extends DataConnection implements DMXConnection{
	public static final int HANDLE_SENDCOMMAND=1;
	private String ipAddress=null;
	private int port=2145;
	private ConnectionThread cThread;
	private Handler uiHandler;
	
	public TCPConnection(Handler appHandler){
		uiHandler=appHandler;
	}
	
	public void setupConnection(String ip, int port){
		ipAddress=ip;
		this.port=port;
	}
	
	/*
	@Override
	public String toString(){
		return "[established] DMXControl Connection to:";
	}
	*/
	
	public void sendCommand(String command){
		Message m=cThread.threadHandler.obtainMessage(HANDLE_SENDCOMMAND);
		m.getData().putString("command",command);
		m.sendToTarget();
	}
	
	public synchronized void connect(){
		//if connected, disconnect
		if(ipAddress==null){
			Log.d("DMX","dmxconn was not properly initialized");
			return;
		}
		
		if(isConnected()){
			Log.d("DMX","connection still active, doing nothing");
			//disconnect();
			return;
		}
		
		cThread=new ConnectionThread(ipAddress, port);
		cThread.setRunning(true);
		cThread.start();
	}
	
	public void disconnect(){
		//stop thread
		//join
		if(cThread==null||!cThread.getRunning()){
			return;
		}
		Log.d("DMX","closing connection");
		boolean retry = true;
		cThread.setRunning(false);
		while(retry){
			try {
				cThread.join();
                retry=false;
			}
			catch(InterruptedException e){
				Log.d("DMX","stopping the thread was interrupted");
			}
		}
	}
	
	public boolean isConnected(){
		return cThread!=null&&cThread.getRunning();
	}
	
	protected void finalize(){
		disconnect();
	}
	
	private class ConnectionThread extends Thread{
		private Socket s;
		private SocketAddress sa;
		private String ip;
		private int port;
		private boolean isRunning=false;
		private OutputStream out;
		private InputStream in;
		private String readBack=new String("");
		private byte[] readBuffer;
		
		public Handler threadHandler=new Handler(){
			public void handleMessage(Message msg){
				switch(msg.what){
					case HANDLE_SENDCOMMAND:
						if(msg.getData().containsKey("command")){
							write(msg.getData().getString("command"));
						}
						break;
					default:
						break;
				}
			}
		};
		
		public ConnectionThread(String ip, int port){
			this.ip=ip;
			this.port=port;
		}
		
		public synchronized void setRunning(boolean b){
			isRunning=b;
		}
		
		public boolean getRunning(){
			return isRunning;
		}
		
		public synchronized void write(String data){
			try{
				out.write((data+"\n").getBytes());
        		Log.d("DMX","sent: "+data);
			}
			catch(Exception e){
				Log.d("DMX","exception while writing");
			}
		}
		
		public void run(){
			s=new Socket();
        	try{
        		s.setSoLinger(true,3);
        		s.setSoTimeout(5000);
        	}
        	catch(Exception e){
        		Log.d("DMX","failed setting timeouts");
        		Message m=uiHandler.obtainMessage(0,RemoteApp.HANDLE.DISPLAYTOAST);
        		m.getData().putString("message","Warning: No timeouts set");
        		m.sendToTarget();
        	}
        	
        	sa=new InetSocketAddress(ip,Integer.valueOf(port));
        	try{
        		s.connect(sa,3000);
        		out=s.getOutputStream();
        		in=s.getInputStream();
        	}
        	catch(Exception e){
        		isRunning=false;
        		Log.d("DMX","failed to connect");
        		
        		Message m=uiHandler.obtainMessage(0,RemoteApp.HANDLE.DISPLAYTOAST);
        		m.getData().putString("message","Connection failed\nCheck your settings");
        		m.sendToTarget();
        		
        		m=uiHandler.obtainMessage(0,RemoteApp.HANDLE.CONNECTIONFAILED);
        		m.getData().putString("message",e.toString());
        		m.sendToTarget();
        		
        		return;
        	}
        	
        	Log.d("DMX","entering loop");
        	Message m=uiHandler.obtainMessage(0,RemoteApp.HANDLE.DISPLAYTOAST);
    		m.getData().putString("message","Connection OK");
    		m.sendToTarget();
    		
    		m=uiHandler.obtainMessage(0,RemoteApp.HANDLE.CONNECTIONOK);
    		m.getData().putString("message","Connection OK");
    		m.sendToTarget();
    		
        	while(isRunning){
        		try {
        			int dataReady=in.available();
        			if(dataReady>0){
        				Log.d("DMX",dataReady+" bytes to read");
        				readBuffer=new byte[dataReady];
        				//FIXME UGLY HACK
        				//BAD DESIGN HAS TO BE COUNTERED WITH MOAR BAD DESIGN. OH WELL.
        				in.read(readBuffer,0,dataReady);
        				String temp=new String(readBuffer);
        				readBack=new String(readBack+temp);
        				Log.d("DMX","read buffer: "+readBack);
        				try{
	        				if(readBack.charAt(0)=='C'&&readBack.charAt(readBack.length()-1)=='\n'){
	        					Log.d("DMX","detected complete oneliner");
	        					//last command was a one-liner and is done
	        					//post to handler
	        					readBack=readBack.substring(0,readBack.length()-1);
	        	        		m=uiHandler.obtainMessage(0,RemoteApp.HANDLE.DATAREADY);
	        	        		m.getData().putString("data",readBack);
	        	        		m.sendToTarget();
	        					//reset string
	        					readBack=new String("");
	        				}
	        				else if(readBack.charAt(0)!='C'&&readBack.regionMatches(readBack.length()-3,"\n.\n",0,3)){
	        					Log.d("DMX","detcted complete multiliner");
	        					//last command was a multi-liner and is done
	        					//post to handler
	        					readBack=readBack.substring(0,readBack.length()-3);
	        					m=uiHandler.obtainMessage(0,RemoteApp.HANDLE.DATAREADY);
	        	        		m.getData().putString("data",readBack);
	        	        		m.sendToTarget();
	        					//reset string
	        					readBack=new String("");
	        				}
        				}
        				catch(Exception e){
        					Log.d("DMX","caught "+e.toString());
        				}
        			}
					sleep(1);//give up time slice
				}
        		catch(InterruptedException e){}
        		catch(IOException e){
        			Log.d("DMX","IOEXCEPTION! CANT READ! WE R DOOM'D");
        		}
        	}
        	try{
				s.close();
			} 
        	catch(IOException e) {}
		}
		
	}

	@Override
	public void requestDeviceList() {
		sendCommand("GetAllDevices");
	}

	@Override
	public void requestCommandsList() {
		sendCommand("GetCommandPossibilities 1");
	}
	
	@Override
	public void setChannel(int channel,int value){
		//todo bounds checking
		sendCommand("SC "+channel+" "+value);
	}

	@Override
	public void sendArbitraryCommand(String command) {
		sendCommand(command);
	}

	@Override
	public boolean isReady() {
		return isConnected();
	}

	@Override
	public void start() {
		connect();		
	}

	@Override
	public void stop() {
		disconnect();
	}

	@Override
	public void requestDeviceInfo(int deviceID) {
		sendCommand("GetDeviceInfo "+deviceID);
	}

	@Override
	void write(String what) {
			cThread.write(what);
	}
	
}
